#include <iostream>
//#include "Polymer.h"
#include "Polymers.h"
//#include "Vector3d.h"
#include "defs.h"


Polymers::Polymers(int NumMon, double StepLength, double Repulsion, int NumChains, double boxwidth, long seed, double SoftCore, double HardCore, SimulationType simtype ) {
   vect_f boxsize(boxwidth,boxwidth,boxwidth);
   Polymers(NumMon, StepLength, Repulsion, NumChains,boxsize, seed, SoftCore, HardCore, simtype);
}

Polymers::Polymers(int NumMon, double StepLength, double Repulsion, int NumChains, double * boxsize_array, long seed, double SoftCore, double HardCore, char * sim_name) {

    vect_f boxsize(boxsize_array[0],boxsize_array[1],boxsize_array[2]);
    SimulationType simtype;

    if (!strcmp(sim_name,"Monte Carlo")) {simtype = MonteCarloType;
    } 
    else if (!strcmp(sim_name,"Runge Kutta")) {simtype = RungeKuttaType;
    }

   Polymers(NumMon, StepLength, Repulsion, NumChains,boxsize, seed, SoftCore, HardCore, simtype);
}

Polymers::Polymers(int NumMon) {
   Polymers(16,1.0F,0.1F,1,16.0F,12345L,2.0F,1.0F,RungeKuttaType);
}

Polymers::Polymers(int NumMon, double StepLength, double Repulsion, int NumChains, vect_f boxsize, long seed, double SoftCore, double HardCore, SimulationType simtype) {
   this->NumMon = NumMon;
   this->StepLength = StepLength;
   this->seed = seed;
   this->Repulsion = Repulsion;
   this->NumChains = NumChains;
   TotMon = NumMon*NumChains;
   Rand = new RandNumGenerator(seed);
   polymers = new Polymer[NumChains];
   if (simtype == RungeKuttaType) {
      MommaCoords = new vect_f[2*TotMon];
      MommaCopy = new vect_f[2*TotMon];
      setCoordsVelo(MommaCoords);//Should this be here?
   } else {
      MommaCoords = new vect_f[TotMon];
      setCoords(MommaCoords);//Should this be here?
   }
   makePolymerMonIndex();
   setCores(SoftCore,HardCore);
   if (boxsize*boxsize > 0) setGrid(boxsize);
   for(int i = 0; i < NumChains; i++)
   {
      polymers[i].set(NumMon, StepLength, Repulsion, MommaCoords + i*NumMon); 
      polymers[i].setPolymers(this);
   }
}

void Polymers::makeBinders(double ComingOnRate, double ComingOffRate) {
   for(int i=0; i < NumChains; i++) {
      MyosinBinding * Bind1 = new MyosinBinding(ComingOnRate, ComingOffRate, RK->getStepSize(), Rand);
      MyosinBinding * Bind2 = new MyosinBinding(ComingOnRate, ComingOffRate, RK->getStepSize(), Rand);
      polymers[i].putBinders(Bind1, Bind2);
   }
}

vect_f * Polymers::getMommaCoords() {
   return MommaCoords;
}

void Polymers::setCoords(vect_f * mommacoords) {

   for(int i = 0; i < NumChains; i++) {
      polymers[i].setCoords(mommacoords + i*NumMon); 
   }
}

void Polymers::setNNConstant(double NNConstant) {
   this->NNConstant = NNConstant;

   for(int i = 0; i < NumChains; i++)
   {
      polymers[i].setNNConstant(NNConstant);
   }
}


void Polymers::setSpringConstant(double SpringConstant) {
   this->SpringConstant = SpringConstant;

   for(int i = 0; i < NumChains; i++)
   {
      polymers[i].setSpringConstant(SpringConstant);
   }
}

void Polymers::setSpringConstantArray(double * SpringConstantArray) {
   this->SpringConstantArray = SpringConstantArray;

   for(int i = 0; i < NumChains; i++)
   {
      polymers[i].setSpringConstantArray (SpringConstantArray);
   }
}

void Polymers::setExtForce(double ExtForce) {
   this->ExtForce = ExtForce;

   for(int i = 0; i < NumChains; i++)
   {
      polymers[i].setExtForce(ExtForce);
   }
}

void Polymers::setCoordsVelo(vect_f * mommacoords) {

   for(int i = 0; i < NumChains; i++) {
      polymers[i].setCoords(mommacoords + i*NumMon); 
      polymers[i].setVelocities(mommacoords + TotMon+ i*NumMon); 
   }
}

void Polymers::initVelocities() {
   for(int i=0; i < TotMon; i++) {
      vect_f g; 
//#define SCRUNCHED
#ifdef SCRUNCHED
      double r = Rand->nextGaussian();
      g.set(0.0,0.0,r);
#else
      Rand->getGaussVect_f(1.0,g);
      Rand->getGaussVect_f(1.0,g);
#endif
      MommaCoords[TotMon+i] = g;
   }
   vect_f Vave = 0;
   for(int i=0; i < TotMon; i++) {
      Vave += MommaCoords[TotMon+i];
   }
   Vave /= TotMon;
   for(int i=0; i < TotMon; i++) {
      MommaCoords[TotMon+i] -= Vave;
   }
}

   void Polymers::setSimulationRungeKutta(double StepSize,double damping) {
   erasePolymerGrid(MommaCoords);
   SimType = RungeKuttaType;
   Noise = new vect_f[TotMon];
   setDamping(StepSize, damping);
   RK = new RungeKutta(NumMon*NumChains*2, StepSize);
   RK->setRKable((RKable *) this);

   for(int i = 0; i < NumChains; i++) {
      polymers[i].tension = new Tension(NumMon-1, StepLength, Damping);//linear chain has NumMon-1 links
   }
   initVelocities();
   adjustChain();

}

void Polymers::adjustChain() {
   if (Damping != 0) {
      return;
   }
   vect_f L;
   totalAngMom(L); 
   subAngMom(L);
   zeroMomentum();
   vect_f half_boxsize = 0.5*grid->getBoxWidth();
   shiftCoords(half_boxsize);
   NormalizeKineticEnergy();
}

void Polymers::shiftCoords(vect_f shift) {
   vect_f COM;
   centerOfMass(COM);
   for(int i=0; i < TotMon; i++) {
      MommaCoords[i] += shift - COM;
   }
}

void Polymers::setDamping(double StepSize, double damping){
   Damping = damping;
   NoiseAmplitude = sqrt(2.0*damping/StepSize);//damping*(Temperature=1), also m=1
}

void Polymers::setInitKE() {
   NormalizeKineticEnergy();
   TotalKinEnergInit  = totalKineticEnergy();
}


vect_f *  Polymers::CopyMommaCoords(vect_f * CoordsCopy) {
   for(int i = 0; i < TotMon; i++) {
      CoordsCopy[i] = MommaCoords[i];
   }
   return CoordsCopy;
}

void  Polymers::CopyMommaCoords() {
   CopyMommaCoords(MommaCopy);
}

Polymers * Polymers::clone() {//using same Rand, so as to get different random #'s for clone
   //vect_f zero(0.0,0.0,0.0);
   //Polymers * polys = new Polymers(NumMon, StepLength, Repulsion, NumChains, zero, seed, SoftCoreRadius, HardCoreRadius, SimType);
   Polymers * polys = new Polymers(NumMon, StepLength, Repulsion, NumChains, grid->getBoxWidth(), seed, SoftCoreRadius, HardCoreRadius, SimType);
   for(int i = 0; i < NumChains; i++) {
      vect_f * coords = polymers[i].getCoords();
      polys->polymers[i].setCoords(coords);
   }
   polys->populatePolymerGrid(); //may want to include but will take space to include
   polys->makeConfiningPotential(WellDepth, WellWidth);
   return polys;
}

void Polymers::setRepulsion(double Rep) {
   Repulsion = Rep;
   for(int i = 0; i < NumChains; i++) {
      polymers[i].Repulsion = Repulsion; 
   }
}


int  Polymers::getNumChains() const{
   return NumChains;
}

int  Polymers::getNumMon() const {
   return NumMon;
}

int  Polymers::getTotMon() const {
   return TotMon;
}

double Polymers::getStepLength() const{
   return StepLength;
}


Polymer * Polymers::getPolymers() {
   return polymers;
}

double Polymers::getHardCore() {
   return HardCoreRadius;
}


GridPeriodic * Polymers::getGrid() {
   return grid;
}


void Polymers::setCores(double SoftCoreRadius, double HardCoreRadius) {
   this->SoftCoreRadius = SoftCoreRadius;
   this->HardCoreRadius = HardCoreRadius;
   for(int i = 0; i < NumChains; i++)
   {
      polymers[i].setCores(SoftCoreRadius, HardCoreRadius);
   }
}

void Polymers::makePolymerMonIndex(){
   PolymerIndex  = new int[NumMon*NumChains];
   MonomerIndex  = new int[NumMon*NumChains];
   for(int i=0; i < NumChains; i++) {
      for(int j=0; j < NumMon; j++) {
	 PolymerIndex[i*NumMon + j] = i;
	 MonomerIndex[i*NumMon + j] = j;
      }
   }
}

void Polymers::initPutChain(int ChainNum, double **initR) {
    polymers[ChainNum].initPutChain(initR);
}

void Polymers::makeRandPolys() {
   for(int i=0; i < NumChains; i++) {
      vect_f pos;
      vect_f boxsize = grid->getBoxWidth();
      pos.set(boxsize.xcomp()*Rand->nextFloat(),boxsize.ycomp()*Rand->nextFloat(),boxsize.zcomp()*Rand->nextFloat());
      polymers[i].initRand(pos, boxsize);
   }
}

void Polymers::makeScrunchedPolys() {
   for(int i=0; i < NumChains; i++) {
      vect_f pos;
      vect_f boxsize = grid->getBoxWidth();
      pos.set(boxsize.xcomp()*Rand->nextFloat(),boxsize.ycomp()*Rand->nextFloat(),boxsize.zcomp()*Rand->nextFloat());
      polymers[i].initScrunched(pos);
   }
}


int Polymers::getMonomer(int const indx) const {
   return MonomerIndex[indx];
}

int Polymers::getPolymer(int const indx) const {
   return PolymerIndex[indx];
}
/*
double ** Polymers::getPythonPolymers() const {
    double ** out_array = new double*[NumChains*NumMon];
    for(int i=0; i < 3; i++) {
        out_array[i] = new double[3];
    }
    for(int n=0; n < NumMon*NumChains; n++) {
        vect_f coord = MommaCoords[n];
        out_array[n][0] = coord.xcomp();
        out_array[n][1] = coord.ycomp();
        out_array[n][2] = coord.zcomp();
    }
    return out_array;
}
*/

 double * Polymers::getPythonPolymers() const {
    double * out_array = new double[3*NumChains*NumMon];

    for(int n=0; n < NumMon*NumChains; n++) {
        vect_f coord = MommaCoords[n];
        out_array[3*n+0] = coord.xcomp();
        out_array[3*n+1] = coord.ycomp();
        out_array[3*n+2] = coord.zcomp();
    }
    return out_array;
}           

void  Polymers::setPythonPolymers(double * in_array) {
    for(int n=0; n < NumMon*NumChains; n++) {
	MommaCoords[n].set(in_array[3*n+0],in_array[3*n+1],in_array[3*n+2]);
    }
}           

RandNumGenerator * Polymers::getRandGen() {
   return Rand;
}

void Polymers::setGrid(vect_f boxsize) {
   grid = new GridPeriodic(boxsize, SoftCoreRadius);
   grid->set_max_radius(SoftCoreRadius);
}

void Polymers::setGrid(double boxwidth) {
   vect_f BoxWidth = boxwidth*(i_hat + j_hat + k_hat);
   setGrid(BoxWidth);
}

int Polymers::fixMove(int RepTime,int GigTime) {
   int num_accepted = 0;
   for(int i = 0; i < RepTime; i++) {
      int WhichPolymer = Rand->nextInt(NumChains);
      Polymer * polymer = polymers + WhichPolymer;
      num_accepted += polymer->reptate_once();
   }
   for(int i = 0; i < NumChains; i++) {
      Polymer * polymer = polymers + i;
      polymer->reshuffle();
   }
   for(int i = 0; i < GigTime; i++) {
      int WhichPolymer = Rand->nextInt(NumChains);
      Polymer * polymer = polymers + WhichPolymer;
      num_accepted += polymer->jiggle_once();
   }
   return num_accepted;
}

double Polymers::RgSq() {
   double rgsq = 0;
   for(int i=0; i < NumChains; i++) {
      rgsq += polymers[i].RgSq();
   }
   return rgsq/NumChains;
}

double Polymers::REndEndSq() {
   double end_end_sq = 0;
   for(int i=0; i < NumChains; i++) {
      end_end_sq += polymers[i].REndEndSq();
   }
   return end_end_sq/NumChains;
}

void Polymers::zThickness(double & z_ave, double & var) {
   for(int i=0; i < NumChains; i++) {
      for(int j=0; j < NumMon; j++) {
	 double z = polymers[i].Coords[j].zcomp();
	 z_ave += z;
	 var += z*z;
      }
   }
   int NumPoints = (NumMon*NumChains);
   z_ave /= (NumPoints);
   var = (var/NumPoints) - z_ave*z_ave;
}


void Polymers::makeConfiningPotential(double WellDepth, double WellWidth){
   VConfining = new double[(int)((grid->getBoxWidth()).zcomp()/WellWidth)+1];
   VConfining[0] = 1e10;
   VConfining[1] = WellDepth;
   this->WellWidth  = WellWidth;
}

vect_f Polymers::getMax() {
    vect_f max_coord = -1e30*(x_hat+y_hat+z_hat);
    for(int i=0; i < TotMon; i++) {
        max_coord.setX(max(MommaCoords[i].xcomp(), max_coord.xcomp()));
        max_coord.setY(max(MommaCoords[i].ycomp(), max_coord.ycomp()));
        max_coord.setZ(max(MommaCoords[i].zcomp(), max_coord.zcomp()));
    }

    return max_coord;

}

vect_f Polymers::getMin() {
    vect_f min_coord = 1e30*(x_hat+y_hat+z_hat);
    for(int i=0; i < TotMon; i++) {
        min_coord.setX(min(MommaCoords[i].xcomp(), min_coord.xcomp()));
        min_coord.setY(min(MommaCoords[i].ycomp(), min_coord.ycomp()));
        min_coord.setZ(min(MommaCoords[i].zcomp(), min_coord.zcomp()));
    }
    
    return min_coord;

}


void Polymers::centerOfMass(vect_f &C_O_M) {
   C_O_M.set(0.F,0.F,0.F);
   for(int i=0; i < TotMon; i++) {
      C_O_M += MommaCoords[i];
   }
   C_O_M /= TotMon;
}

void Polymers::totalAngMom(vect_f &L) {
   L.set(0.F,0.F,0.F);
   vect_f COM;
   centerOfMass(COM);

   for(int i=0; i < TotMon; i++) {
      vect_f delta = MommaCoords[i]-COM;
      L += delta ^ MommaCoords[i+TotMon];
      }
}

void gaussj(double **a, int n, double **b, int m);

void Polymers::subAngMom(vect_f Lt) {
   vect_f * L = new vect_f[4];
   vect_f Dir[4];
   vect_f COM;
   centerOfMass(COM);

   Dir[1] = x_hat;
   Dir[2] = y_hat;
   Dir[3] = z_hat;

   for(int dir=1; dir < 4; dir++) {
      for(int i=0; i < TotMon; i++) {
	 vect_f delta = MommaCoords[i]-COM;
	 L[dir] += delta^(delta^Dir[dir]);
      }
   }


   double ** M = new double *[4];
   double ** rhs = new double *[4];
   for (int i=0; i < 4; i++) {
      M[i] = new double[4];
   }
   for (int i=0; i < 4; i++) {
      rhs[i] = new double[2];
   }

   rhs[1][1] = Lt.xcomp();
   rhs[2][1] = Lt.ycomp();
   rhs[3][1] = Lt.zcomp();

   for(int i=1; i < 4; i++) {
      M[i][1] = L[i].xcomp();
      M[i][2] = L[i].ycomp();
      M[i][3] = L[i].zcomp();
   }

   gaussj(M,3,rhs,1);

   vect_f omega(rhs[1][1],rhs[2][1],rhs[3][1]);

   for(int i=0; i < TotMon; i++){
      vect_f delta = MommaCoords[i]-COM;
      MommaCoords[TotMon+i] -= (delta^omega);
   }

   vect_f newL;
   totalAngMom(newL);
   cout << "total L = " << newL <<endl;

   delete [] L;

}



double Polymers::totalEnergy() {
   double tot_e = 0.0;
   double tot_conf = 0.0;

   for(int chain_num=0; chain_num < NumChains; chain_num++) {
      Polymer * poly = polymers + chain_num;
      for(int j=0; j < NumMon; j++) {
	 vect_f point = polymers[chain_num].Coords[j];
	 tot_e +=  poly->ePointAndChains(point, j);
	 tot_conf += VConfining[(int)(point.zcomp()/WellWidth)];
      }
   }
   return 0.5*tot_e + tot_conf;
}

void Polymers::erasePolymerGrid(vect_f * MommaCopy) {
   for (int i=0; i < NumChains; i++) {
      //Polymer poly = polymers[i];
      for(int j=0; j < NumMon; j++) {
	 removeMonomer(&(polymers[i]), j, MommaCopy[j+i*NumMon]);
      }
   }
//   grid->isGridEmpty();
}


void Polymers::populatePolymerGrid() {
   for (int i=0; i < NumChains; i++) {
      //Polymer poly = polymers[i];
      for(int j=0; j < NumMon; j++) {
	 polymers[i].insert(j);
      }
   }
}



/******** Runge Kutta simulation code *********/


   void Polymers::derivs (double t, vect_f * y, vect_f * dydt){
   int i,j;

   //   setCoords...

   //for debugging:
   //grid->isGridEmpty();
  
   setCoordsVelo(y);//set each Polymer pointer to those from y
#ifndef PHANTOM_CHAIN
   populatePolymerGrid();
   //CopyMommaCoords();
#endif


#if 0//test for NumMon = 2, NumChains = 1
   //double T = (SQR(y[2]) + SQR(y[3]));
   double T = (SQR(y[2]) + SQR(y[3]))/y[0].distsq(y[1]);
   //double T = .553634644F;

   dydt[0] = y[2];
   dydt[1] = y[3];
   dydt[2] = T * (y[1]-y[0]);
   dydt[3] = -1.0*dydt[2];

#else 
   vect_f f_end;//force exerted ON tethered end
   for(i=0; i < NumChains; i++) {
      vect_f * force = polymers[i].getForce(t,Noise + NumMon*i,f_end);
      for(j=0; j < NumMon; j++){
	 int ind = NumMon*i + j;
	 //r^.
	 dydt[ind] = y[ind+TotMon];
	 //v^.
	 if (Damping > 0) {
	    dydt[ind + TotMon] = force[j] - Damping*dydt[ind];// + Noise[ind];
	 } else {
	    dydt[ind + TotMon] = force[j];
	 }

      }
#ifdef SPRING_SIMULATION
   delete[] force;
#endif
   }
#endif
#ifndef PHANTOM_CHAIN
   //erasePolymerGrid(y);
   erasePolymerGrid(y);
#endif
}

void Polymers::makeNoise() {
   for(int i=0; i < TotMon; i++) {
      Rand->getGaussVect_f(NoiseAmplitude,Noise[i]);
   }
}

void Polymers::move(int n_steps) {
#ifndef PHANTOM_CHAIN
//   erasePolymerGrid();
#endif
   for(int i=0; i < n_steps;i++) {
      if (Damping > 0) {
	 makeNoise();
      }
#ifdef MUSCLE_SIMULATION
      bool b1 = polymers[0].Bind1->TryToChangeState();
      bool b2 = polymers[0].Bind2->TryToChangeState();

      if (b1) {
	 cout << "bind1 changed state to is bound = "  << polymers[0].Bind1->isBound() << endl;
      }
      if (b2) {
	 cout << "bind2 changed state to is bound = "   << polymers[0].Bind2->isBound() << endl;
      }
#endif
      RK->rkpp(i*RK->h,MommaCoords,MommaCoords);
      //RK->DumbUpdate(i*RK->h,MommaCoords,MommaCoords);
   }
     // if (0 && i%100==0) 

   NormalizeKineticEnergy();
#ifndef PHANTOM_CHAIN
//   populatePolymerGrid();
#endif
}

void Polymers::PrintErrors() {
   cout << "arclength error =" << (double)errorArcLength() << endl;
   if (Damping == 0) {
      cout << "Kinetic Energy error = " << 1-totalKineticEnergy()/TotalKinEnergInit << endl;
      vect_f COM,L;
      centerOfMass(COM);
      cout <<  "Center of Mass = " << COM << endl;
      totalAngMom(L); 
      cout <<  "Ang Mom = " << L << endl;
   }
}

double Polymers::totalKineticEnergy() {
   double TotE=0;
   for(int i= TotMon; i < 2*TotMon; i++) {
      TotE += SQR(MommaCoords[i]);
   }
   return TotE*0.5;
}

void Polymers::zeroMomentum() {
   vect_f Vave = 0;
   for(int i=0; i < TotMon; i++) {
      Vave += MommaCoords[TotMon+i];
   }
   Vave /= TotMon;
   for(int i=0; i < TotMon; i++) {
      MommaCoords[TotMon+i] -= Vave;
   }

}

void Polymers::NormalizeKineticEnergy() {
   double Temperature =1;
   int numDOF = NumChains*((NumMon-1)*2 +3);
   double rightKE = 0.5*Temperature*numDOF;
   double totalKE = totalKineticEnergy();
   double norm = sqrt(rightKE/totalKE);
   for(int i=0; i < TotMon;i++) {
      MommaCoords[i+TotMon] *= norm;
   }
}

double Polymers::totalArcLength() {
   double TotLength = 0;
   for(int i=0; i < NumChains; i++) {
      vect_f * coords  = polymers[i].Coords;
      for(int j=1; j < NumMon; j++) {
	 TotLength += sqrt(coords[j].distsq(coords[j-1]));
      }
   }
   return TotLength;
}

double Polymers::errorArcLength() {
   return 1-(totalArcLength()/((NumMon-1)*StepLength*NumChains));
}

