#ifndef POLYMERS_H
#define POLYMERS_H
#include "GridPeriodic.h"
#include "RandNumGenerator.h"
#include "RungeKutta.h"
#include "SimTypeDef.h"
//#include "Polymer.h"

using namespace std;


class Polymer;

class Polymers : public RKable {
   friend class Polymer;
   protected:
   GridPeriodic * grid;
   RandNumGenerator * Rand;
   SimulationType SimType;
   vect_f * MommaCoords;//all the polymer coords for all the chains in a flat array
   vect_f * MommaVelocities;//all the polymer velocities for all the chains in a flat array
   vect_f * Noise;
   vect_f * MommaCopy;//copy of all coords
   double actin_velocity;
   int NumMon; 
   int NumChains; 
   int TotMon;
   double StepLength; 
   double Repulsion; 
   double SpringConstant;
   double *SpringConstantArray;
   double NNConstant;
   double ExtForce;
   double SoftCoreRadius;
   double HardCoreRadius;
   double WellDepth;
   double WellWidth;
   double * VConfining;
   double TotalKinEnergInit;
   double Damping;
   double NoiseAmplitude;
   long seed;
   Polymer * polymers;
   int * PolymerIndex, * MonomerIndex;//translate index in grid to the corresponding polymer and monomer.
   //RungeKutta<vect_f> * RK;
   RungeKutta * RK;
   public:
   Polymers(int NumMon, double StepLength, double Repulsion, int NumChains, double boxwidth, long seed, double SoftCore, double HardCore, SimulationType simtype); 
   Polymers(int NumMon, double StepLength, double Repulsion, int NumChains, vect_f boxsize, long seed, double SoftCore, double HardCore, SimulationType simtype); 
   Polymers(int NumMon, double StepLength, double Repulsion, int NumChains, double * boxsize_array, long seed, double SoftCore, double HardCore, char * sim_name);
   Polymers(int NumMon);
   void removeMonomer(Polymer * polymer, int Monomer, vect_f pos);
   void addMonomer(Polymer * polymer, int Monomer, vect_f pos);
   int getMonomer(int const indx) const;
   int getPolymer(int const indx) const;
   int getNumChains() const;
   int getNumMon() const;
   int  getTotMon() const;
   double getHardCore();
   void zThickness(double & z_ave, double & var);
   double getStepLength() const;
   Polymer * getPolymers();
   GridPeriodic * getGrid();
   void makePolymerMonIndex();
   void setGrid(double boxwidth);
   void setGrid(vect_f boxsize);
   void setCores(double SoftCoreRadius, double HardCoreRadius);
   void setSpringConstant(double SpringConstant);
   void setSpringConstantArray(double * SpringConstant);
   void setNNConstant(double NNConstant);
   void setExtForce(double ExtForce);
   void setActinVelocity(double actin_velocity);
   RandNumGenerator * getRandGen();
   int fixMove(int num_steps) ;
   int fixMove(int RepTime,int GigTime);
   void makeRandPolys();
   void makeScrunchedPolys();
   double RgSq();
   double REndEndSq();
   void makeConfiningPotential(double WellDepth, double WellWidth);
   void setRepulsion(double Rep);
   virtual void setSimulationRungeKutta(double StepSize,double damping);
   double totalEnergy();
   //void move(int n_steps); 
   void move(int NumItsDone, int n_steps);
   void setCoordsVelo(vect_f * mommacoords);
   void setCoords(vect_f * mommacoords);
   vect_f * getMommaCoords();
   virtual void derivs (double t, vect_f * y, vect_f * dydt);
   void erasePolymerGrid(vect_f * MommaCopy);
   void populatePolymerGrid();
   void initVelocities();
   double totalArcLength();
   double errorArcLength();
   double totalKineticEnergy();
   Polymers * clone();
   void centerOfMass(vect_f &C_O_M);
   void totalAngMom(vect_f &L);
   void subAngMom(vect_f Lt);
   void setInitKE();
   void NormalizeKineticEnergy();
   void PrintErrors();
   void makeNoise();
   double * getPythonPolymers() const ;
   void  setPythonPolymers(double * in_array) ;
   void setDamping(double StepSize, double damping);
   void shiftCoords(vect_f shift);
   vect_f *  CopyMommaCoords(vect_f * CoordsCopy);
   void CopyMommaCoords();
   void adjustChain();
   void zeroMomentum();
   vect_f getMin();
   vect_f getMax();
   void setCollision(bool CollisionSim);
   void setCollisionVelocity(double CollVelo);
   void setTotalKinEnergyInit(double TotalKinEnergInit);
   void setTotalKinEnergyInit();
   void makeBinders(double ComingOnRate, double ComingOffRate);
   void initPutChain(int ChainNum, double **initR);
   double getActinVelocity();
};


#include "Polymer.h"

   inline void Polymers::addMonomer(Polymer * polymer, int Monomer, vect_f pos) {
   int poly_num = polymer-polymers;
   int indx = poly_num*NumMon + Monomer;
   vect_i box;
   grid->get_box(pos, box);
   grid->put(box, indx);
}

   inline void Polymers::removeMonomer(Polymer * polymer, int Monomer, vect_f pos) {
   int poly_num = polymer-polymers;
   int indx = poly_num*NumMon + Monomer;
   vect_i box;
   grid->get_box(pos, box);
   grid->erase(box, indx);
}


extern "C" {
    double ** set_muscle_params(double Height, int num_its_done, Polymers * Ps, double  SpringConstant, double  NNConstant,  double ExtForce, double  BindingAngle, double  VActinDepth, double  ActinSpacing, double ComingOnRate, double ComingOffRate, double RK_StepSize, double Damping, int NumMoves, double ** r, double * SpringConstantArray, double actin_velocity, double * fe);
    void freePolys(Polymers * Ps);
    }
#endif// POLYMERS_H
