#include <iostream>
#include "Polymers.h"
#include "defs.h"
#include "PolymersDamped.h"


void PolymersDamped::setSimulationRungeKutta(double StepSize,double damping) {
   erasePolymerGrid(MommaCoords);
   SimType = RungeKuttaType;
   Noise = new vect_f[TotMon];
   setDamping(StepSize, damping);
   RK = new RungeKutta(NumMon*NumChains*2, StepSize);
   RK->setRKable((RKable *) this);

   for(int i = 0; i < NumChains; i++) {
      polymers[i].tension = new TensionDamped(NumMon-1, StepLength, Damping);//linear chain has NumMon-1 links
   }
   initVelocities();
   adjustChain();

}

/******** Runge Kutta simulation code *********/


void PolymersDamped::derivs (double t, vect_f * y, vect_f * dydt){
   int i,j;
  
#ifndef PHANTOM_CHAIN
   populatePolymerGrid();
#endif

   for(i=0; i < NumChains; i++) {
      vect_f f;
      vect_f * force = polymers[i].getForce(t,0,f);
      for(j=0; j < NumMon; j++){
	 int ind = NumMon*i + j;
	    dydt[ind] = force[j] + Noise[ind];
	 }
#ifdef SPRING_SIMULATION
   delete[] force;
#endif
      }
#ifndef PHANTOM_CHAIN
   erasePolymerGrid(y);
#endif
}

