

from scipy import *
from pylab import *
import pylab
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import cm
import matplotlib.pyplot as plt
import numpy as np
import matplotlib.ticker as ticker



#
#DT = 0.1
DT = 0.5

#this causes the array p to diffuse (blur) a little:
def diffuse(p, dt= DT):
    p[1:-1] =  (1-2*dt)*p[1:-1] + dt*(p[0:-2] + p[2:])
    return



#The number of points in the potential array v:
nx = 100


#The v array:
v = zeros((nx))
v_old = zeros((nx))

#Lets you analyze more precisedly what happens on a smaller scale by rescaling the x's by this factor:
length_scale = 0.2
#a constant that appears in the evolution and is precalculated to speed things up a little:
c_l = length_scale**2
#The real range of x is scaled by length_scale:
x = arange(0,nx)*length_scale
#The total amount of fixed charge. The equation below deals with densities, so this says: integral density dx = Q0
Q0 = -10.0


ion()


for k in range(50000):
    #The 0 needs to be here to really copy v to v_old, otherwise it just copies the pointers
    v_old = v + 0
    #first diffuse
    diffuse(v)
    #then add the effect of the mobile charges
    v -= c_l*DT*sinh(v)
#   Linearized eqn:
#    v -= c_l*DT*v

#   keep the boundaries at 0 potential
    v[0] = 0
    v[-1] = 0
#   now subtract off the effect of the fixed charge in the middle:
    v[nx/2] -= length_scale*Q0*DT

#   graph it occasionally (if you graph it too much it'll really slow things down)
    if k%50 == 0: 
#   clear the screen:
       cla()
       plot(x,v)
       draw()
       pause(0.0001)
#      check how much the solution has changed between iterations:
       error = sqrt(sum((v_old-v)**2))
       print "error = ", error
       if error < 1e-5:
           break


#Now compare it with the analytical solution for the linearized case:
plot(x,-0.5*Q0*exp(-abs(x-length_scale*nx/2)))
draw()
show()
pause(100)


