
from scipy import *
from scipy.linalg import *
from pylab import *
import scipy.integrate as integrate

class Tension:
    def __init__(self, T, n, d):
        self.T = T
        self.delta = zeros((n-1,d),dtype=double)
        self.delta_f = zeros((n-1,d),dtype=double)
        self.u = zeros((n-1),dtype=double)
        self.l = zeros((n-1),dtype=double)
        self.rhs = zeros((n-1,d),dtype=double)
        self.diag = zeros((n-1),dtype=double)
        self.force = zeros((n,d),dtype=double)


    def tension(self, r, f):
        self.delta = r[1:,:]-r[:-1,:]
        self.diag = -2*sum(self.delta*self.delta,axis=1)
        self.delta_f = f[1:,:]-f[:-1,:]
        u_vect = self.delta[1:,:]*self.delta[:-1,:]
        self.u[1:] = sum(u_vect, axis = 1)
        self.l[:-1] = self.u[1:]
        self.rhs = -sum(self.delta_f*self.delta,axis=1) - c1*(-0.5*self.diag - StepLengthSq)

        return solve_banded((1, 1), array((self.u,self.diag,self.l)), self.rhs, overwrite_ab=0, overwrite_b=0, debug=0)

    def calcForce(self,r,f):
        t = self.tension(r,f)
        for i in range(d):
            self.force[1:-1,i] = t[1:]*self.delta[1:,i] - t[0:-1]*self.delta[0:-1,i] + f[1:-1,i]
        self.force[0,:] = t[0]*self.delta[0,:] + f[0,:]
        self.force[n-1,:] = -t[n-2]*self.delta[n-2,:] + f[n-1,:]
        return self.force


def StiffForces(r):
    f_stiff[2:n-3,:] =  StiffnessConst*(2*r[2:n-3,:]-r[4:n-1,:] - r[0:n-5,:])
    f_stiff[1,:] = StiffnessConst*(r[1,:] - r[3,:])
    f_stiff[0,:] = StiffnessConst*(r[0,:] - r[2,:])
    f_stiff[n-2,:] = StiffnessConst*(r[n-2,:] - r[n-4,:])
    f_stiff[n-1,:] = StiffnessConst*(r[n-1,:] - r[n-3,:])
    return f_stiff

seed(3002)

r0 = 4/sqrt(2.0)
v_0 = 5*2

def bind_force(ra,rb):
    deltar = rb - ra
    return 4*v_0*r0**2*deltar/(r0**2+sum(deltar*deltar))**2

def dx_dt(r_in,t):
    r = reshape(r_in,(n,d))
    f[:,:] = noise[:,:] + StiffForces(r)
    b = bind_force(r[na,:],r[nb,:])
    f[na,:] += b
    f[nb,:] -= b
    f[0,0] += -f_ext
    f[n-1,0] += f_ext
    return tens.calcForce(r,f).flatten()

def length_error(r):
    delta = r[1:,:] - r[:-1,:]
    dlt_sq = delta*delta
    return sum(sum(dlt_sq)) - (n-1)

def construct_chain(n):
    r = zeros((1,d))
    r1 = zeros((1,d))
    for i in range(n-1):
        dlt = standard_normal((d))
        dlt = StepLength*dlt/linalg.norm(dlt)
        r1 += reshape(dlt,(1,d))
        
        r = append(r, r1, 0)
    return r

   

StepLength = 1.0
StepLengthSq = StepLength**2
d = 3
n = 16
na = int(n/4)
nb = int(3*n/4)
dt = 0.1
#T = 2.00
T = 1.00
#f_ext = 2.9
f_ext = 2.95
#f_ext = 3.0
c1 = 1 #helps to maintain constant link lengths
f_stiff = zeros((n,d),dtype=double)
StiffnessConst = 1.0

tens = Tension(T, n, d)

r = construct_chain(n)

t = arange(0,10*dt,dt)

f = zeros((n,d))

graphics= True
if graphics:
    from vpython import *

    def vc(r):
        if d== 3:
           return vector(r[0],r[1],r[2])
        elif d==2:
           return vector(r[0],r[1],0)



    ball = []
    cyl = []
    for i in range(n-1):
        cyl.append(cylinder (pos=vc(r[i]), axis=vc(r[i+1]-r[i]), color=color.red, radius = 0.2))

    floor = box (pos=vector(0,-4,-4), length=n, height=0.5, width=4, color=color.blue)

    balla = sphere(pos= vc(r[na]),color=color.blue,radius = 0.4)
    ballb = sphere(pos= vc(r[nb]),color=color.green,radius = 0.4)

    
if graphics:
    ion()


x_out = r.flatten()

x_out = reshape(x_out,(1,n*d))

display_interval = 20

fmax = 6
time = 0;
#In this case we don't increment the external force but watch the chain fluctuate at a fixed valud of f_ext
f_incr = 0.00
f_list = []
dist_list = []
tot_time = 600
for t_coarse in arange(0,tot_time,dt):
    if graphics:
       rate(10000)
    t = array([t_coarse,t_coarse+dt])
    noise = sqrt(2*T/dt)*standard_normal((n,d))
    x_out = integrate.odeint(dx_dt, x_out[-1] , t, full_output=False)
    time += 1

    if time%300 == 0 : 
        print(length_error(reshape(x_out[-1],(n,d))))
    if time%display_interval == 0 : 
        r_out = reshape(x_out[-1],(n,d))
        if  graphics:
            for i in range(n-1):
                cyl[i].pos = vc(r_out[i])
                cyl[i].axis = vc(r_out[i+1]-r_out[i])
            balla.pos = vc(r_out[na])
            ballb.pos = vc(r_out[nb])
        print("f_ext = ", f_ext)
        delta = r_out[nb]-r_out[na]
        f_list.append(f_ext)
        dist_list.append(sqrt(sum(delta*delta)))

    if f_ext > fmax:
        f_incr = -f_incr
    if f_ext < 0:
        break
    f_ext += f_incr


figure(0)
xlabel("f")
ylabel("dist")
plot(f_list,dist_list)
draw()
show()
figure(1)
xlabel("t")
ylabel("dist")
plot(dist_list)
draw()
show()
figure(2)
#N, bins, patches = hist(dist_list, 20, normed=1)
N, bins, patches = hist(dist_list, 20, density=1)
draw()
show()


