from scipy import *
from pylab import *

seed(2002)#set seed for random number generator so it produces a fixed sequence of random numbers

d=3#dimension of space the simulation will be run in.

if d==2:
    dir = array([ [1,0], [-1,0], [0,1], [0,-1]])
if d==3:
    dir = array([ [1,0,0], [-1,0,0], [0,1,0], [0,-1,0],[0,0,1],[0,0,-1]])


def randdir():
    n = randint(2*d)
    return dir[n]


n=32
if d==2:
    dim = (n,n)
    center = (n//2,n//2)
if d==3:
    dim = (n,n,n)
    center = (n//2,n//2,n//2)

p = zeros(dim)

#The reptation dynamics algorithm. Chain slithers like a snake but backwards and forwards randomly
#with the new position of the head and tail being chosen randomly by the Metropolis algorithm
#reptate_interval is the number of reptation moves attempted
def reptate(r, reptate_interval):
    head = length-1
    for i in range(0,reptate_interval):
        WhichEnd = randint(2)
        HeadOrTail = (head + WhichEnd)%length
        TailOrHead = (head + 1-WhichEnd)%length
        r_new = r[HeadOrTail,:] + randdir()
        p[tuple(r[TailOrHead,:])] -= 1
        if p[tuple(r_new)] >= 1:
           # move failed so reset:
           p[tuple(r[TailOrHead,:])] += 1
        else:
           e1 = nn_energy(p,r[TailOrHead])
           e2 = nn_energy(p,r_new)
           de = -J*(e2-e1)
           if rand() > exp(-de): #The Metropolis algorithm which obeys detailed balance
           # move failed so reset:
                 p[tuple(r[TailOrHead,:])] += 1
           else:
                 head = (head - 2 * WhichEnd + 1 + length)%length
                 r[(head + WhichEnd)%length,:] = r_new
                 p[tuple(r_new)] = 1

    tail = (head + 1 + length)%length
    r_tmp[length-tail:,:] =  r[:tail,:]
    r_tmp[:length-tail,:] = r[tail:,:]
    r[:,:] = r_tmp[:,:]


def new_chain(length):#construct a new chain as a random walk. It'll self-intersect
    r = zeros((length,d), dtype = int)
    r[0] = center
    p[tuple(r[0,:])] = 1
    for i in range(1,length):
        while 1:
            r_try =  r[i-1,:] + randdir()
            if p[tuple(r_try)] <= 1:
                break
        r[i,:] = r_try
        p[tuple(r[i,:])] = 1
    return r

def nn_energy(p,pos):#energy between monomers and its nearest neighbors
    e = 0
    for dr in dir: #search over all nearest neighbors to count occupied sites
        if p[tuple(pos+dr)] == 1:
            e += 1
    return e

#Radius of gyration squared
def Rgsq(r):
    rcm = sum(r,axis=0)*1.0/length
    return sum((r-rcm)**2)*1/length

 
#Confine the chain to a box by putting in barriers at the faces:
if d == 2:
    p[:,0] = p[:,-1] = p[0,:] = p[-1,:] = 2
if d == 3:
    p[:,:,0] = p[:,:,-1] = p[0,:,:] = p[-1,:,:] = p[:,0,:] = p[:,-1,:] = 2
   

length = 64
r_tmp = zeros((length,d), dtype = int)#temporary storage of coordinates
r = new_chain(length)#r stores the monomer coordinates 
#nearest neighbor attraction 
#J = 0.2693 #the critical point (theta point)
#J = 0.5
J = 0.0

graphics= True
if graphics:
    from vpython import *

    def vc(r):
        if d== 3:
           return vector(r[0],r[1],r[2])
        elif d==2:
           return vector(r[0],r[1],0)


    ball = []
    cyl = []
    for i in range(length-1):
        cyl.append(cylinder (pos=vc(r[i]), axis=vc(r[i+1]-r[i]), color=color.red, radius = 0.2))

#floor = box (pos=(center[0],center[1],-4), length=n, height=0.5, width=4, color=color.blue)

display_interval = 1

reptate_interval = length*length//2
#reptate_interval = 100
ion()


rsq_tot = 0
num_meas = 0
ready = False

print("sum p = ", sum(p))
for j in range(0,20000):
    if graphics:
        rate(10000)
    reptate(r,reptate_interval)
    if not ready:#because the initial configuration overlaps, we move it until the chain doesn't overlap:
        sum_p = sum(p)
        if j%1 == 0:
            print("sum p = ", sum_p, "should = ", 2*(n**d-(n-2)**d) + length)
        if sum_p ==  2*(n**d-(n-2)**d) + length:
            ready = True
            print("equilibrated, starting measurements")
    if ready:
        rsq_tot += Rgsq(r)
        num_meas += 1
        print("num meas = ", num_meas, " radius of gyration = ", sqrt(rsq_tot/num_meas))
    if j%display_interval == 0 : 
        if  graphics:
            for k in range(length-1):
                cyl[k].pos = vc(r[k])
                cyl[k].axis = vc(r[k+1]-r[k])

print("sum p = ", sum(p), "should = ", 2*(n**d-(n-2)**d) + length)


    

