
from scipy import *
from pylab import *
from scipy.integrate import *


Cm=1
g_K =  36.0
g_Na = 120.0
g_l =  0.3
V_Na = -115.0
VK = 12.0
Vl = -10.613

def I(Vt, V, m, n, h):
    return Cm*Vt + g_K*(n**4)*(V-VK)+ g_Na*(m**3)*h*(V-V_Na)+ g_l*(V-Vl)



def alpha_n(V, a_0 = 0.01, V_n = 10.0, V_shift = 10.0):
    return a_0*(V+V_shift)/(exp((V+V_shift)/V_n)-1.0)


def beta_n(V, b_0 = 0.125, V_N = 80.0):
    return b_0*exp(V/V_N)


def alpha_m(V):
    return  alpha_n(V, a_0 = 0.1, V_n = 10.0, V_shift = 25.0)

def beta_m(V):
    return beta_n(V,b_0 = 4.0, V_N = 18)

def alpha_h(V):
    return beta_n(V, b_0=0.07, V_N=20)

V_shift_h = 30
v_n = 10.0
def beta_h(V):
    return 1.0/(exp((V+V_shift_h)/v_n)+1.0)

def lin_interpolate(arr, begin,end,init,fin):
    slope = (fin-init)/(end-begin)
    for x in range(0,end-begin):
        arr[begin+x] = init + x*slope
    return

tp = 50

#Vmax = 110
#Vmax = 15 #below threshold
Vmax = 20  #above threshold
length = 500
V0 = 0
V=V0*ones((length))
lin_interpolate(V,0,5,0,-Vmax)
lin_interpolate(V,5,10,-Vmax,20)
lin_interpolate(V,10,400,20.,0)
V_old = zeros((length))
n0 = alpha_n(V0)/(alpha_n(V0)+beta_n(V0))
m0 = alpha_m(V0)/(alpha_m(V0)+beta_m(V0))
h0 = alpha_h(V0)/(alpha_h(V0)+beta_h(V0))
h = h0*ones((length))
m = m0*ones((length))
n = n0*ones((length))
total_time = 30.0


num_steps = 100000

dt = 0.01
D = 10.0
pos = []
figure(0)
ion()
for i in range(num_steps):
    V_old[1:-1] =  (1-D*dt)*V[1:-1] + dt*0.5*D*(V[2:]+V[:-2]) 
    V_old[0] =  (1-D*dt)*V[0] + dt*0.5*D*(V[1]+V[-1]) 
    V_old[-1] = (1-D*dt)*V[-1] + dt*0.5*D*(V[-2]+V[0]) 
    V = V_old - dt*(g_K*(n**4)*(V-VK)+ g_Na*(m**3)*h*(V-V_Na)+ g_l*(V-Vl))
    n += dt*(alpha_n(V)*(1.0-n)-beta_n(V)*n)
    m += dt*(alpha_m(V)*(1-m)-beta_m(V)*m)
    h += dt*(alpha_h(V)*(1-h)-beta_h(V)*h)
    pulse_pos = argmin(V[0:length//2])
    pos.append(pulse_pos)
    if i%40 == 0:
        pause(0.001)
       
        print(i, pulse_pos)
        cla()
        plot(V,label='V')
        plot(10*m,label='m')
        plot(10*n,label='n')
        plot(10*h,label='h')
        xlabel("x")
        legend()
        draw()
        pause(0.0001)
    if pulse_pos > length/2 -50:
        break

figure(1)
plot(dt*arange(0,i+1),pos)
title('pulse peak x vs t')
xlabel("t")
ylabel("x")
draw()

show()
pause(100)
print("done")


