#include <stdio.h>
#include <math.h>
#include <unistd.h>
#include "graphics_on.h"
#include "Corr.h"
#include "defs.h"
#ifdef GRAPHICS_ON
#include "graphics.h"
#endif

static System  * sys;
static int window;
static int stop=0;
static int its = 0;

#ifdef GRAPHICS_ON
GLfloat mat_ambient_red[] = { 0.8, 0.0, 0.0, 1.0 };
GLfloat mat_ambient_green[] = { 0.0, 0.8, 0.0, 1.0 };
GLfloat mat_ambient[] = { 0.5, 0.5, 0.5, 1.0 };
GLfloat **  mat_ambient_colors;
vect_f cntr = 0;
GLfloat size = 0;
#endif

void init_idle (System  * s)
{
   sys = s;
#ifdef GRAPHICS_ON
   int num_colors = 16;
   size_2d(mat_ambient_colors,num_colors,4);
   for(int i=0; i <num_colors; i++)
   {
      double r = (i%2)/3.;
      double g = (i%3)/4.;
      double b = (i%5)/6.;
      mat_ambient_colors[i][0] = r + 0.1;
      mat_ambient_colors[i][1] = g + 0.1;
      mat_ambient_colors[i][2] = b + 0.05;
      mat_ambient_colors[i][3] = 1;
   }
#endif
}

#ifdef GRAPHICS_ON
void register_window(int win)
{
   window = win;
}

void display(void)
{
   int i, chain_num;
   System * s = sys;
   double eps = 0.1;
   Polymers * polymers = sys->getPolymers(); 
   int NumChains = polymers->getNumChains();
   int NumMon = polymers->getNumMon();
   double a = 0.5 * polymers->getHardCore();
   double r[3];
   double r1[3];
   glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   glLoadIdentity();

   glMatrixMode(GL_MODELVIEW);

   vect_f max_coord  = polymers->getMax();
   vect_f min_coord  = polymers->getMin();
   if (size == 0) {
       size = 2* sqrt(max_coord.distsq(min_coord));
       cntr =0.5*(max_coord+min_coord);
   } 
   else {
       size = (1-eps)*size + eps* 2 * sqrt(max_coord.distsq(min_coord));
       cntr = (1-eps)*cntr + eps*0.5*(max_coord+min_coord);
   }
//   cout << size << "," << cntr << endl;


   gluLookAt(  cntr.xcomp(),cntr.ycomp(),cntr.zcomp()+size,  //  eyepos

                    cntr.xcomp(),cntr.ycomp(),cntr.zcomp(),   //  aimpoint

                                    0,1,0); //  up direction

   glShadeModel (GL_SMOOTH);

   glPushMatrix();
   for(chain_num=0; chain_num < NumChains; chain_num++)
   {
      Polymer * polymer = polymers->getPolymers()+chain_num;
      vect_f * coords =   polymer->getCoords();
      for(i=0; i < NumMon;i++)
      {

	 r[0] = coords[i].xcomp();
	 r[1] = coords[i].ycomp();
	 r[2] = coords[i].zcomp();
	 glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient_colors[chain_num%16]);
	 graph_sphere(r, 0.1);
	 //glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
      }
   glPopMatrix();
   glPushMatrix();
      bool  b[2]; 
      polymer->isMyosinBound(b);
      if (b[0]) {
	 r[0] = coords[1].xcomp();
	 r[1] = coords[1].ycomp();
	 r[2] = coords[1].zcomp();
	 glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient_colors[3]);
	 graph_sphere(r, 0.2);
      }
      if (b[1]) {
	 r[0] = coords[NumMon-2].xcomp();
	 r[1] = coords[NumMon-2].ycomp();
	 r[2] = coords[NumMon-2].zcomp();
	 glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient_colors[3]);
	 graph_sphere(r, 0.2);
      }
   glPopMatrix();
   }

   glPushMatrix();
   Polymer * poly = polymers->getPolymers();
   double act = poly->getActinSpacing();
   int lower = (cntr.xcomp() - size - act)/act;
   int upper = (cntr.xcomp() + size + act)/act;
   for (i= lower; i < upper; i++) {
      r[0] = (i+0.5)*poly->getActinSpacing();
      r[1] = r[2] = 0;
      glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient_colors[8]);
      graph_sphere(r, 0.2);
   }

	 glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);

   glPopMatrix();
   glPushMatrix();
   for(chain_num=0; chain_num < NumChains; chain_num++)
   {
      Polymer * polymer = polymers->getPolymers()+chain_num;
      vect_f * coords =   polymer->getCoords();
      for(i=0; i < NumMon - 1;i++)
      {

	 r[0] = coords[i].xcomp();
	 r[1] = coords[i].ycomp();
	 r[2] = coords[i].zcomp();
	 r1[0] = coords[i+1].xcomp();
	 r1[1] = coords[i+1].ycomp();
	 r1[2] = coords[i+1].zcomp();

	 graph_cylinder( r, r1,  0.1);
      }
   }
   glPopMatrix();
//   glFlush();
   glutSwapBuffers();

}


void idle_test()
{
//   sleep(1);
   glutPostRedisplay();
}

#if 0
int change_window_coords(SYSTEM * sys)
{
   PARAMS * params = (PARAMS *) sys->params;
   CHAIN_STATS * stats = params->stats;
   static CHAIN_STATS old_stats = {0};
   double size= old_stats.size;
   double max_size= 0, min_size= 1e30;
   int i;
   int dim = params->dimension;
   int change_window = 0;

   if (old_stats.min == 0)
   {
      calloc_(old_stats.min, dim);
      calloc_(old_stats.max, dim);
      calloc_(old_stats.com, dim);
      calloc_(old_stats.std_dev, dim);
   }

   for(i=0; i < dim; i++) 
   {
      double delta = stats->max[i] - stats->min[i];
      max_size = max(max_size, delta);
      min_size = min(min_size, delta);
   }

   if (max_size > size || max_size < size/8.0)
   {
      size = max_size*4;
      change_window = 1;
   }

   for(i=0; i < dim; i++) 
   {
      //if (stats->min[i] < old_stats.min[i] || stats->max[i] > old_stats.max[i])
      if (stats->min[i] < old_stats.com[i] - size*0.5  || stats->max[i] > old_stats.com[i] + size*0.5)
	 change_window = 1;
   }


   if (change_window)
   {
      stats->size = size;
      old_stats = stats[0];
   }

   return change_window;
}

void reshape (int w, int h)
{
   PARAMS * params = (PARAMS *) sys[0]->params;
   CHAIN_STATS * stats = params->stats;
   double * com = stats->com;
   GLfloat size = 0.5  * stats->size;


   glViewport(0, 0, (GLsizei) w, (GLsizei) h);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   if (w <= h)
      glOrtho(-size + com[0], size + com[0], -size*(GLfloat)h/(GLfloat)w + com[1],
         size*(GLfloat)h/(GLfloat)w + com[1], -(size * 4 + com[2]), -(-size*4 + com[2]));
   else
      glOrtho(com[0] + -size*(GLfloat)w/(GLfloat)h,
         com[0] + size*(GLfloat)w/(GLfloat)h, com[1] - size, com[1] + size, -(com[2] + size * 4), -(com[2] -size*4));

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
}

void reshape (int w, int h)
{
   //double size=sys->pm->x_size;
   Polymers *  polymers = sys->getPolymers();
   GridPeriodic * grid = polymers->getGrid();
   double size = grid->getBoxWidth().xcomp();
   glViewport(0, 0, (GLsizei) w, (GLsizei) h);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
#if 0
   if (w <= h)
      glOrtho(-size, size, -size*(GLfloat)h/(GLfloat)w,
         size*(GLfloat)h/(GLfloat)w, -size*2, size * 2);
   else
      glOrtho(-size*(GLfloat)w/(GLfloat)h,
         size*(GLfloat)w/(GLfloat)h, -size, size, -size*2, size * 2);
#else
   if (w <= h)
      glOrtho(-.1*size, size, -.1*size,
         size*(GLfloat)h/(GLfloat)w, -size*4, size * 4);
   else
      glOrtho(-.1*size,
         size*(GLfloat)w/(GLfloat)h, -.1*size, size, -size*4, size * 4);
#endif
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
#if 0
   gluPerspective( /* field of view in degree */ 40.0,
  /* aspect ratio */ 1.0,
    /* Z near */ 1.0, /* Z far */ 100.0);
#endif
  glMatrixMode(GL_MODELVIEW);
#if 1
  gluLookAt(10.0, 10.0, 60.0,  0.0, 0.0, 0.0,      0.0, 1.0, 0.);      
#else
  gluLookAt(30.0, 30.0, 30.0,  /* eye is at (0,0,30) */
    0.0, -10.0, 10.0,      /* center is at (0,0,0) */
    0.0, 1.0, 0.);      /* up is in positive Y direction */
#endif




  glPushMatrix();       /* dummy push so we can pop on model
                           recalc */


}
#endif

void reshape(int w, int h) {

    if (h==0) h=1;

    // set the drawable region of the window

    glViewport(0,0,w,h);



    // set up the projection matrix 

    glMatrixMode(GL_PROJECTION);

    glLoadIdentity(); 

    // just use a perspective projection

    gluPerspective(45,(float)w/h,0.1,1000);

    // go back to modelview matrix so we can move the objects about

    glMatrixMode(GL_MODELVIEW);
}


void keyboard(unsigned char key, int x, int y)
{
   switch (key) 
   {
      case 27:
	 exit(0);
	 break;
      case 'q':
	 exit(0);
	 break;
      case 'p':
//	 sleep(1);
	 break;
      case 'P':
//	 sleep(10);
	 break;
      case 's':
	 stop ^= 1;
	 break;
   }

}

#endif
void idle()
{
   int i, finished=0;
   Polymers * polymers = sys->getPolymers();
//   Corr * corr = sys->getCorr();

      //sleep(1);
   if (stop)
   {
//      sleep(1);
   }
   else
   {
      {
	 //printf("idle(): starting move()...");
         //polymers->fixMove(sys->getGigTime(),sys->getGigTime());
         polymers->move(sys->getGigTime());
	 its++;
	 if (its%100 == 1) 
	 {
	    polymers->PrintErrors();
	    cout << "Num Its = " << its << endl;
	 }
	 //printf("completed\n");
//	 corr->update();
#ifdef GRAPHICS_ON
	 if (sys->getGraphOn())
	 {
	    glutSetWindow(window);
	    glutPostRedisplay();
	 }
#endif
	 sys->incrItsDone();
	 if (sys->getNumItsDone() < sys->getNumIts())
	    finished=0;
	 else
	    finished = 1;
      }

   }
   if (finished == 1)
   {
//      corr->printStatsCout();
//      corr->printStats();
//      corr->printHists();
      sys->getPolymers()->PrintErrors();
      cout << "# rem3d visits = " << sys->getPolymers()->getGrid()->getRem_Count() << endl;
#if 0
      FILE * corr_out = fopen("corr_out.d","w");
      FILE * p_k_out = fopen("p_k_out.d","w");
      FILE *rho_out = fopen("rho_out.d","w");
      print_corr(sys->pm, stdout ,corr);
      print_corr(sys->pm, corr_out ,corr);
      print_p_k(sys->pm, p_k_out, corr);
      print_rho(sys->pm, rho_out, corr);
      print_stats(sys->pm, stdout ,corr);
      free_system(sys);
#endif
//      corr->ofClose();

      printf("simulation complete\n");
      system("date");
      //exit(0);
      sys->setDone();
   }


}

