//FIX DAMPING!
#include <iostream>
//#include <mcheck.h>
#include <unistd.h>
#include <assert.h>
#include "graphics_on.h"
#include "Corr.h"
#include "argv.h"
#ifdef GRAPHICS_ON
#include "graphics.h"
#endif
#include "SimTypeDef.h"

using namespace std;

#define MEM_DEBUG
//#define GRAPHICS
//FIX DAMPING!

extern "C" {

    double ** set_muscle_params(double Height, int num_its_done, Polymers * Ps, double  SpringConstant, double  NNConstant,  double ExtForce, double  BindingAngle, double  VActinDepth, double  ActinSpacing, double ComingOnRate, double ComingOffRate, double RK_StepSize, double Damping, int NumMoves, double ** r, double * SpringConstantArray, double actin_velocity, double * fe) {
        double Repulsion = 0.0;
        int NumMon  = Ps->getNumMon(); 
        Ps->setSpringConstant(SpringConstant);
        Ps->setNNConstant(NNConstant);
        Ps->setExtForce(ExtForce);
        Ps->setActinVelocity(actin_velocity);

        Polymer * poly = Ps->getPolymers();
        vect_f actin_dir;
        actin_dir.set(cos(BindingAngle),sin(BindingAngle),0);
        poly->setActinParams(VActinDepth, actin_dir, ActinSpacing);
        poly->setHeight(Height);
        vect_f  pos;
        pos.set(0.5*poly->getActinSpacing(),0.0,0.0);
        Ps->setRepulsion(Repulsion);
        Ps->setSpringConstantArray(SpringConstantArray);
//        poly->initRand(pos,Ps->getGrid()->getBoxWidth(), poly->getActinDir());
        Ps->initPutChain(0,r);
        Ps->setSimulationRungeKutta(RK_StepSize,Damping);
        Ps->makeBinders(ComingOnRate, ComingOffRate);
        Ps->move(num_its_done,NumMoves);
	{
	   vect_f * coords =  poly->getCoords();
           *fe = coords[2*NumMon].xcomp()/(RK_StepSize*NumMoves);
	   int i,j;
	   for (i=0; i < NumMon; i++)
	   {
	      r[i][0] = coords[i].xcomp();
	      r[i][1] = coords[i].ycomp();
	      r[i][2] = coords[i].zcomp();
	   }

	   //freePolys(Ps);
	 
	   return r;
	}

    }
#if 0
    void freePolys(Polymers * Ps) {
      int NumMon  = Ps->getNumMon(); 
      Polymer * poly = Ps->getPolymers();
      for(int j=0; j < NumMon;j++)
	      poly->remove(j);
//      Ps->getGrid()->freeGrid();
    }
#endif
}

#if 0
int main(int argc, const char ** argv) {
#ifndef GRAPHICS_ON
   void idle();
#endif
   void init_idle (System  * s);
#if 1
   SimulationType simtype;
   int NumChains = 8;
   double StepLength = 1.0;
   double Repulsion = -1.0;
   double SoftCore = -1.0;
   double HardCore = -0.5;
   double actin_velocity = 0;
   int RepLength = -1;
   int NumIts = -1;
   int GigTime = -1;
   int EquilTime = -1;
   int NumGTimeSteps = -1;
   bool GraphOn = false;
   bool ZeroMass = false;
   double WellDepth = 0;
   double WellWidth = 0;
   const char * DensityName;
   const char * StatsName;
   const char * DirName = 0;
   int NumWallGridPoints = -5;
   double MiddleSep = -1.0;
   double LayerSize = -1.0;
   double SpringConstant = 1.0;
   double Height = 0;
   double BindingAngle = M_PI/4;
#endif
    double * spring_consts;
    double ComingOnRate = 0.01;
    double ComingOffRate = 0.0001;
    double Damping = 1.0;
    double RK_StepSize = 0.005;
    double VActinDepth = 15.0;
    double ActinSpacing = 4.0;
    double NNConstant = 0.0;
    double ExtForce = 0.0;
    int   ChainLength = 12;
    int  BoxSize = 16;
    long  RanSeed = 12345;
    int x_size = BoxSize;
    int y_size = BoxSize;
    int z_size = BoxSize;
    int NumMoves = 1000;

      assert(system("date") != -1);
      for(int i=1; i < argc;) {
	 int j=0;
	 if (j = ParseArgv(argv+i+j, &ChainLength,"-chain","len")){}
	 else if (j = ParseArgv(argv+i+j, &NumChains,"-num","chain")){}
	 else if (j = ParseArgv(argv+i+j, &BoxSize,"-box","siz")){}
	 else if (j = ParseArgv(argv+i+j, &x_size,"-x","siz")){}
	 else if (j = ParseArgv(argv+i+j, &y_size,"-y","siz")){}
	 else if (j = ParseArgv(argv+i+j, &z_size,"-z","siz")){}
	 else if (j = ParseArgv(argv+i+j, &RanSeed,"-ran","seed")){}
	 else if (j = ParseArgv(argv+i+j, &StepLength,"-step","len")){}
	 else if (j = ParseArgv(argv+i+j, &Height,"-hei","t")){}
	 else if (j = ParseArgv(argv+i+j, &actin_velocity,"-act","vel")){}
	 else if (j = ParseArgv(argv+i+j, &Repulsion,"-rep","uls")){}
	 else if (j = ParseArgv(argv+i+j, &SpringConstant,"-spri","const")){}
	 else if (j = ParseArgv(argv+i+j, &NNConstant,"-nn","const")){}
	 else if (j = ParseArgv(argv+i+j, &SoftCore,"-soft","core")){}
	 else if (j = ParseArgv(argv+i+j, &HardCore,"-hard","core")){}
	 else if (j = ParseArgv(argv+i+j, &ExtForce,"-ext","force")){}
	 else if (j = ParseArgv(argv+i+j, &RepLength,"-rep","len")){}
	 else if (j = ParseArgv(argv+i+j, &NumIts,"-num","its")){}
	 else if (j = ParseArgv(argv+i+j, &GigTime,"-gig","tim")){}
	 else if (j = ParseArgv(argv+i+j, &EquilTime,"-eq","tim")){}
	 else if (j = ParseArgv(argv+i+j, &NumGTimeSteps,"-num","gtime")){}
	 else if (j = ParseArgv(argv+i+j, &GraphOn,"-graph","on")){}
	 else if (j = ParseArgv(argv+i+j, &ZeroMass,"-zero","mas")){}
	 else if (j = ParseArgv(argv+i+j, &WellDepth,"-wel","dept")){}
	 else if (j = ParseArgv(argv+i+j, &WellWidth,"-wel","wid")){}
	 else if (j = ParseArgv(argv+i+j, &DensityName,"-density","out")){}
	 else if (j = ParseArgv(argv+i+j, &StatsName,"-stats","out")){}
	 else if (j = ParseArgv(argv+i+j, &DirName,"-dir","name")){}
	 else if (j = ParseArgv(argv+i+j, &NumWallGridPoints,"-num","wall")){}
	 else if (j = ParseArgv(argv+i+j, &MiddleSep,"-mid","sep")){}
	 else if (j = ParseArgv(argv+i+j, &LayerSize,"-layer","size")){}
	 else if (j = ParseArgv(argv+i+j, &RK_StepSize,"-step","size")){simtype=RungeKuttaType;}
	 else if (j = ParseArgv(argv+i+j, &Damping,"-dam","ng")){simtype=RungeKuttaType;}
	 else if (j = ParseArgv(argv+i+j, &ComingOnRate,"-coming","on")){}
	 else if (j = ParseArgv(argv+i+j, &ComingOffRate,"-coming","off")){}
	 else if (j = ParseArgv(argv+i+j, &VActinDepth,"-vact","depth")){}
	 else if (j = ParseArgv(argv+i+j, &BindingAngle,"-bind","ang")){}
	 else if (j = ParseArgv(argv+i+j, &ActinSpacing,"-act","spac")){}
	 else {
	    cout << "didn't understand command line\n" <<  argv[i] << "\n" <<  endl;
	    exit(0);
	 }
	 i += j;
      }

      double ** initR;
    calloc_(spring_consts, ChainLength);
    size_2d(initR,  ChainLength , 3);
    for(int i =0; i < ChainLength-1; i++)
    {
        initR[i+1][0] += initR[i][0] + StepLength;
        spring_consts[i] = SpringConstant;
    }



#ifndef GRAPHICS_ON
   GraphOn = false;
#endif

   Polymers * Ps;
   if (ZeroMass) {
      Damping = 1.0;
      if (x_size == 0 || y_size == 0 || z_size == 0) {
	 Ps = (PolymersDamped *) new PolymersDamped(ChainLength, StepLength, 0, NumChains, BoxSize, -RanSeed, SoftCore,HardCore,simtype);
      }
      else {
	 vect_f BoxSizeVect(x_size,y_size,z_size);
	 Ps = (PolymersDamped *) new PolymersDamped(ChainLength, StepLength, 0, NumChains, BoxSizeVect, -RanSeed, SoftCore,HardCore,simtype);
      }
   }
   else {
      if (x_size == 0 || y_size == 0 || z_size == 0) {
	 Ps = new Polymers(ChainLength, StepLength, 0, NumChains, BoxSize, -RanSeed, SoftCore,HardCore,simtype);
      }
      else {
	 vect_f BoxSizeVect(x_size,y_size,z_size);
	 Ps = new Polymers(ChainLength, StepLength, 0, NumChains, BoxSizeVect, -RanSeed, SoftCore,HardCore,simtype);
      }
   }

   Ps->setSpringConstant(SpringConstant);
   Ps->setNNConstant(NNConstant);
   Ps->setExtForce(ExtForce);

#ifdef MUSCLE_SIMULATION
   Ps->setActinVelocity(0.0);
   Polymer * poly = Ps->getPolymers();
   vect_f actin_dir;
   actin_dir.set(cos(BindingAngle),sin(BindingAngle),0);
   poly->setActinParams(VActinDepth, actin_dir, ActinSpacing);
   poly->setHeight(Height);
   vect_f  pos;
   //pos.set(0.5*poly->getActinSpacing(),0.0,0.0);
   pos.set(0.0,0.0,0.0);
   poly->initRand(pos,Ps->getGrid()->getBoxWidth(), poly->getActinDir());
#else
   Ps->makeRandPolys();
#endif
  // Ps->makeScrunchedPolys();
   Ps->setSimulationRungeKutta(RK_StepSize,Damping);
#ifdef MUSCLE_SIMULATION
   Ps->makeBinders(ComingOnRate, ComingOffRate);
#endif
   if (WellWidth > 0) Ps->makeConfiningPotential(WellDepth, WellWidth);
   System * sys = new System(RepLength, NumIts, GigTime, EquilTime, RK_StepSize, GraphOn);
   sys->setPolymers(Ps);
//   Corr * corr = new Corr(sys, NumGTimeSteps);
//   corr->setHeaderArgs(argc, argv);
#if 0//called from python. No output files
   if (DirName)
      corr->setStreams(DirName,DensityName,StatsName);
   else
      corr->setStreams(DensityName,StatsName);
   sys->setCorr(corr);
    for(int i =0; i < ChainLength-1; i++)
    {
        initR[i][0] = Ps->getMommaCoords()[i].xcomp();
        initR[i][1] = Ps->getMommaCoords()[i].ycomp();
        initR[i][2] = Ps->getMommaCoords()[i].zcomp();
    }
#endif

   set_muscle_params(Ps, -SpringConstant, NNConstant, ExtForce, BindingAngle, VActinDepth, ActinSpacing, ComingOnRate, ComingOffRate, RK_StepSize, Damping, NumMoves, initR, spring_consts);

   Ps->setRepulsion(Repulsion);
   Ps->move(0,sys->getEquilTime());
   Ps->setInitKE();
//   corr->setInitCoords(Ps->getMommaCoords());
   Ps->PrintErrors();
   
   Ps->setActinVelocity(actin_velocity);
   init_idle(sys);

   if (GraphOn) 
   {
#ifdef GRAPHICS_ON
      int window;
      char  ** argv1= new (char *[1]);
      argv1[0] = new char[1];
      argc = 1;
      glutInit(&argc,argv1);
      glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
      glutInitWindowPosition(100, 100);
      glutInitWindowSize(500, 500);

      window = glutCreateWindow("Photo-Polymer Motion");

      // winIdMain = glutCreateWindow("Polymer");
      //init();
      register_window(window);
      glutDisplayFunc(display); 
      glutReshapeFunc(reshape);
      glutKeyboardFunc(keyboard);

  glMatrixMode(GL_PROJECTION);
  gluPerspective( /* field of view in degree */ 40.0,
  /* aspect ratio */ 1.0,
    /* Z near */ 1.0, /* Z far */ 100.0);
  glMatrixMode(GL_MODELVIEW);
  gluLookAt(0.0, 0.0, 30.0,  /* eye is at (0,0,30) */
    0.0, 0.0, 0.0,      /* center is at (0,0,0) */
    0.0, 1.0, 0.);      /* up is in positive Y direction */

    glPushMatrix();       /* dummy push so we can pop on model
                           recalc */

      init();
      glutIdleFunc(idle);

      glutMainLoop();
#endif
   }
   else
   {
      while(1)
      {
	 idle();
	 if (sys->getDone())
	    break;
      }
   }

   GridPeriodic * grid = Ps->getGrid();
   cout << "is grid empty?" << endl;
   grid->isGridEmpty();

#if 0//test for grid bugs
   GridPeriodic * grid = Ps->getGrid();
   grid->printAll();
   cout << "delete grid:\n what's left over?\n"  << endl;

   for(int i=0; i < NumChains; i++)
   {
      Polymer * polymer  = Ps->getPolymers() + i;
      for(int j=0;  j < ChainLength;j++)
	 polymer->remove(j);
   }
   grid->printAll();
#endif
   return 0;
}
#else
int main(int argc, const char ** argv) {

    double * spring_consts;
    double ComingOnRate = 0.5;
    double actin_velocity = 0.0001;
    double ComingOffRate = 0.005;
    double Damping = 1.0;
    double RK_StepSize = 0.005;
    double VActinDepth = 30.0;
    double BindingAngle = M_PI/4;
    double ActinSpacing = 2.0;
    double SpringConstant = -0.5;
    double NNConstant = 0.0;
    double ExtForce = 0.0;
    double StepLength = 1.0;
    double  SoftCore = 2.0;
    double HardCore = 0.0;
    int   ChainLength = 4;
    int     NumChains = 1;
    int  BoxSize = 16;
    long  RanSeed = 12345;
    int x_size = BoxSize;
    int y_size = BoxSize;
    int z_size = BoxSize;
    int NumMoves = 1000;
    double fe=0.0;
    double ** initR;
    calloc_(spring_consts, ChainLength);
    size_2d(initR,  ChainLength , 3);
    for(int i =0; i < ChainLength-1; i++)
    {
        initR[i+1][0] += initR[i][0] + StepLength;
        spring_consts[i] = SpringConstant;
    }


    vect_f BoxSizeVect(x_size,y_size,z_size);
    printf("initR = %f\n", initR[1][0]);
    Polymers * Ps = new Polymers(ChainLength, StepLength, 0, NumChains, BoxSizeVect, -RanSeed, SoftCore,HardCore,RungeKuttaType);
    set_muscle_params(0.0,0,Ps, -SpringConstant, NNConstant, ExtForce, BindingAngle, VActinDepth, ActinSpacing, ComingOnRate, ComingOffRate, RK_StepSize, Damping, NumMoves, initR, spring_consts, actin_velocity,&fe);
    fe =  Ps->getMommaCoords()[2*NumChains*ChainLength].xcomp()/(RK_StepSize*NumMoves);
  set_muscle_params(0.0,0,Ps, -SpringConstant, NNConstant, ExtForce, BindingAngle, VActinDepth, ActinSpacing, ComingOnRate, ComingOffRate, RK_StepSize, Damping, NumMoves, initR, spring_consts, actin_velocity,&fe);
    printf("initR, fend = %f %f\n", initR[1][0], fe);
}
#endif

