
import numpy
from scipy import *
from pylab import *
import time
import weave
from weave import converters
import os

def initRand(InitPos, InitDir):
    R[0,:] = InitPos[:]
    R[1,:] = R[0,:] + StepLength*ActinDir[:]

    for i in range(2,ChainLength):
        dlt = standard_normal((3))
        dlt = StepLength*dlt/linalg.norm(dlt)
        R[i,:] = R[i-1,:] + dlt[:]
    return


def MyosinWeave():
     global curr_dir
   
     code = """
        #line 23 "tst.py"
        Polymers * Ps;
        int i,j;
        double ** r;
        double fe=0.0;
        double * spring_const;
        vect_f BoxSizeVect(BoxSize,BoxSize,BoxSize);
        spring_const = (double *) calloc(ChainLength,sizeof(spring_const[0]));
        r = (double ** ) calloc(ChainLength,sizeof(r[0]));

        for(i=0; i < ChainLength; i++)
            {
               r[i] = (double *) calloc(3,sizeof(r[0][0]));
               spring_const[i] = SpringConstArray(i);
            }

        for(i=0; i < ChainLength; i++)
            for(j=0; j < 3; j++)
                r[i][j] = R(i,j);

        Ps = new Polymers(ChainLength, StepLength, 0, NumChains, BoxSizeVect, RanSeed, SoftCore,HardCore,RungeKuttaType);
        r = set_muscle_params(Height, NumItsDone,Ps, SpringConstant, NNConstant, ExtForce, BindingAngle, VActinDepth, ActinSpacing, ComingOnRate, ComingOffRate, RK_StepSize, Damping,NumMoves,r, spring_const,ActinVelocity, &fe);
        //Fx(0) = Ps->getMommaCoords()[2*NumChains*ChainLength].xcomp()/(RK_StepSize*NumMoves);
        Fx(0) = fe;


        for(i=0; i < ChainLength; i++)
            for(j=0; j < 3; j++)
                R(i,j) = r[i][j];
        for(i=0; i < ChainLength; i++)
            free(r[i]);
        free(spring_const);
            """
     weave.inline(code, 
     ['Height','NumItsDone','Fx','RK_StepSize', 'Damping', 'ExtForce', 'NNConstant', 'SpringConstant', 'ComingOnRate', 'ComingOffRate', 'VActinDepth', 'ActinSpacing', 'BindingAngle', 'ChainLength','StepLength','NumChains','BoxSize','RanSeed','SoftCore','HardCore','R','NumMoves','SpringConstArray','ActinVelocity'], 
     type_converters=converters.blitz, compiler = 'gcc', 
     include_dirs = ['./',curr_dir,'/usr/include/c++/4.2.1/'],
     headers = ['"System.h"'],
     libraries = ['poly','m'], 
     library_dirs=['/usr/lib','./',curr_dir])

     return

curr_dir = os.getcwd()
RK_StepSize = 0.005
ComingOnAveTime = RK_StepSize/0.5
ComingOffAveTime = RK_StepSize/0.005
ComingOnRate = RK_StepSize/ComingOnAveTime
ComingOffRate = RK_StepSize/ComingOffAveTime
Damping = 1.0
VActinDepth = 30.0
BindingAngle = pi/4
ActinSpacing = 2.0
SpringConstant = -0.5
NNConstant = 0.0
ExtForce = 0.0
ChainLength = 4
NumMoves = 200
#RanSeed = 12345
RanSeed = 12346
StepLength = 1.0
NumChains = 1;
BoxSize = 16
SoftCore = 2.0
HardCore = 0.0
Height=1.0
ActinVelocity=0.001

seed(1000)

Fx = zeros((1))
R = zeros((ChainLength,3))
SpringConstArray = SpringConstant * ones(ChainLength)
#SpringConstArray[3] = 0.0

ActinDir = array([cos(BindingAngle),sin(BindingAngle),0.0])
initRand(array([0.5*ActinSpacing,0.,0.]), ActinDir)

graphics= True
ballnum= 4
if graphics:
    from vpython import *

    def vc(r):
           return vector(r[0],r[1],r[2])

    ball = []
    cyl = []
    for i in range(ChainLength-1):
        cyl.append(cylinder (pos=vc(R[i]), axis=vc(R[i+1]-R[i]), color=color.red, radius = 0.2))

    k = 0
    i  = -ballnum
    while (i < ballnum):
       ball.append(sphere (pos=vector(i*ActinSpacing,Height,0), radius=0.3, color=color.green))
       i += 1
       k += 1

    sphere(pos=vector(0.,0,0), radius=0.3, color=color.blue)


if graphics:
    ion()

Ftot = 0.0
display_interval = 1
SimLength = 1000000
for i in range(1,SimLength):
    if graphics:
        rate(10000)
    NumItsDone = (i-1)*NumMoves
    RanSeed =  -int(randint(0,1000000))
    MyosinWeave()
    Ftot += Fx[0]
    if (R[0,1]**2+R[0,2]**2) > (StepLength*ChainLength)**2:
        print("floated too far away!")
        break;
    if abs(R[0,0]) > 30.0:
        print("gone 30 units, done! time = ", RK_StepSize*i)
        break;
    if  i%100 == 0:
        print("force = ", Ftot/i)
    if  i%display_interval == 0 and graphics:
        for j in range(ChainLength-1):
            cyl[j].pos = vc(R[j])
            cyl[j].axis = vc(R[j+1]-R[j])

        d = RK_StepSize*NumMoves*ActinVelocity*i
        lower = int((-ballnum * ActinSpacing - d)/ActinSpacing)
        upper = int((ballnum * ActinSpacing - d)/ActinSpacing)
        k = 0
        while (lower < upper):
           ball[k].pos = vector(lower * ActinSpacing + d,Height,0.0)
           lower += 1
           k += 1





print("You've run it a long time!")


